//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011"
#property link      ""

#include <CustPrevCalculated.mqh>

//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
enum CPCMyIndResultCode
  {
   CPCMyIndRC_NotInitialized,             //   
   CPCMyIndRC_NoData,                     //  
   CPCMyIndRC_NotChanged,                 //   
   CPCMyIndRC_Changed                     //  
  };

//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
struct MyIndBar
  {
   double Val;                          //  
  };

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
class CCustMyInd
  {
protected:
   bool Initialized;                         //         
   bool UsesExternalData;                    //   
   //  
   //----------------------------
   int BarsLimit;                            //    ,     
   //    
   //----------------------------
   uint DataBarsCount;                       //        ,     BarsLimit
   uint DataBarsCalculated;                  //     
   uint DataStartInd;                        //        (  )
   MyIndBar Buf[];                           //     
   uint MyIndDataBarsCalculated;             //    
   //----------------------------
   bool MyIndDataBarsCuttingLeft;            //         PrepareData()
   CCustPrevCalculated *CustPrevCalculated;
   //  
   //----------------------------
   int __Digits;
   double __Point;
   uint PInd(uint I, bool AsSeries) {return( AsSeries?((DataStartInd+DataBarsCount-1-I) % BarsLimit):((DataStartInd+I) % BarsLimit) );}
public:
   //   
   void CCustMyInd() {Initialized = false;}
   void ~CCustMyInd() {if (CustPrevCalculated != NULL) delete CustPrevCalculated;}
   bool ItIsInitialized() {return(Initialized);}
   //   
   bool Init(CCustPrevCalculated *CPC, string Instr, ENUM_TIMEFRAMES TF, int Limit, CPCHistorySynchMode HSM, uchar HS, uint HSMinute);
   //    
   CPCPrepareDataResultCode PrepareData(CPCPrepareDataResultCode resData);   //       
   //    
   CPCMyIndResultCode Calculate();
   //     
   uint GetBarsCount() {return(DataBarsCount);}                   //    
   uint GetBarsCalculated() {return(MyIndDataBarsCalculated);}    //    
   double GetVal(uint shift, bool AsSeries);                      //    Val  
  };
//+------------------------------------------------------------------+
bool CCustMyInd::Init(CCustPrevCalculated *CPC, string Instr, ENUM_TIMEFRAMES TF, int Limit, CPCHistorySynchMode HSM, uchar HS, uint HSMinute)
{
   if (Initialized)
      return(false);
   
   if (CPC == NULL)
     {
      CustPrevCalculated = new CCustPrevCalculated;
      UsesExternalData = false;
     } else {
      CustPrevCalculated = CPC;
      UsesExternalData = true;
     }
   if (CustPrevCalculated.InitData(Instr, TF, Limit, HSM, HS, HSMinute))
     {
      if (ArrayResize(Buf, Limit) == -1)
         return(Initialized);

      BarsLimit = Limit;
      
      __Digits = (int)SymbolInfoInteger(Instr, SYMBOL_DIGITS);
      __Point = SymbolInfoDouble(Instr, SYMBOL_POINT);
      
      Initialized = true;
      return(true);
     } else {
      Initialized = false;
      return(false);
     }
}
//+------------------------------------------------------------------+
CPCPrepareDataResultCode CCustMyInd::PrepareData(CPCPrepareDataResultCode resData)
{
   if (!Initialized)
      return(CPCPDRC_NoData);
   
   //     
   CPCPrepareDataResultCode res = resData;
   DataBarsCount = CustPrevCalculated.GetDataBarsCount();
   DataBarsCalculated = CustPrevCalculated.GetDataBarsCalculated();
   DataStartInd = CustPrevCalculated.GetDataStartInd();
   
   //     ,      
   //   ,     
   if (CustPrevCalculated.GetDataBarsCuttingLeft())
      MyIndDataBarsCuttingLeft = true;
   
   //        
   for (uint i = (DataBarsCalculated == 0)?0:(DataBarsCalculated+1); i < DataBarsCount; i++)
     {
      Buf[PInd(i, false)].Val = EMPTY_VALUE;
     }
   
   //     ,    
   MyIndDataBarsCalculated = MathMin(DataBarsCalculated, MyIndDataBarsCalculated);
   
   return(res);
}
//+------------------------------------------------------------------+
double CCustMyInd::GetVal(uint shift, bool AsSeries)
{
   if ( shift > (DataBarsCount-1) )
      return(EMPTY_VALUE);
   return(Buf[PInd(shift, AsSeries)].Val);
}
//+------------------------------------------------------------------+
CPCMyIndResultCode CCustMyInd::Calculate()
{
   if (!Initialized)
      return(CPCMyIndRC_NotInitialized);
   
   //     ,   
   if (!UsesExternalData)
     {
      CPCPrepareDataResultCode resCPCPD = CustPrevCalculated.PrepareData();
      CPCPrepareDataResultCode resPD = PrepareData(resCPCPD);

      if (resPD == CPCPDRC_NoData)
         return(CPCMyIndRC_NoData);
      if (resPD == CPCPDRC_NoRecountNotRequired)
         return(CPCMyIndRC_NotChanged);
     }
   
   //  -      
   if (DataBarsCount <= 0)
      return(CPCMyIndRC_NoData);
   
   uint BarsForRecalculation;
   if ( DataBarsCalculated != 0 )
      BarsForRecalculation = DataBarsCount - MyIndDataBarsCalculated - 1;
   else
      BarsForRecalculation = DataBarsCount;
   
   //    
   if (MyIndDataBarsCuttingLeft)
     {
      MyIndDataBarsCuttingLeft = false;
     }

   //======================================================
   //========   ===============================
   //======================================================
   CPCMyIndResultCode res = CPCMyIndRC_NotChanged;
   
   for (uint i = (DataBarsCount - BarsForRecalculation - 1); i < DataBarsCount; i++)
     {
      
      //...
      
      res = CPCMyIndRC_Changed;
      MyIndDataBarsCalculated = i + 1; //  ,     
     } // for ()
   
   return(res);
}